// Jjump - 2d Javascript Game Engine
// Author: Jonas Iversen


Jjump = function() {

/*--------------------------------------------------------------------------------------------------------->
*	Utilities Object
*/
	var Utils = {

		// Adds all properties of Obj1 to obj2
		extend: function(obj1, obj2) {
			var attr;

				for(attr in obj2) {
					if(obj2.hasOwnProperty(attr)) {
						obj1[attr] = obj2[attr];
				}
			}
		},

		// Mediator Pattern Constructor 
		mediator: function() {
			var subscribers = {
				any: []
			};

			var on = function(type, fn, context) {
				type = type || 'any';
				fn = typeof fn === 'function' ? fn : context[fn];

				if(typeof subscribers[type] === "undefined") {
					subscribers[type] = [];
				}

				subscribers[type].push({fn: fn, context: context || this});
			};

			var visitSubscribers = function(action, type, arg, context) {
				var pubtype = type || any,
					sub = subscribers[pubtype],
					i,
					max = sub ? sub.length : 0;

				for(i = 0; i < max; i+= 1) {
					if(action == 'publish') {
						sub[i].fn.call(sub[i].context, arg);
					}
					else {
						if(sub[i].fn === arg && sub[i].context == context) {
							sub.splice(i, 1);
						}
					}
				}
			};

			var remove = function(type, fn, context) {
				visitSubscribers('unsubscribe', type, fn, context);
			};

			var fire = function(type, publication) {
				visitSubscribers('publish', type, publication);
			};

			return {
				on: on,
				remove: remove,
				fire: fire
			};
		},

		quadTree: function() {

		},

		objectPool: function() {
			
		}
	};


/*--------------------------------------------------------------------------------------------------------->
*	InfoMessage Object
*/
	var InfoMessage = function() {

		var newMessage = function(type, message) {
			if(type == "critical")
				critical(message);
			if(type == "warning")
				warning(message);
			if(type == "success")
				success(message);
		};

		var critical = function(message) {
			console.log("Error: " + message);
		};

		var warning = function(message) {
			console.log("Warning: " + message);
		};

		var success = function(message) {
			console.log("Success: " + message);
		};

		return {
			"show": newMessage
		};
	}();


/*--------------------------------------------------------------------------------------------------------->
*	Events Controller Object
*/
	var Events = Utils.mediator();


/*--------------------------------------------------------------------------------------------------------->
*	Load Screen Model Constructor
*/
	var LoadScreen = function() {

	};


/*--------------------------------------------------------------------------------------------------------->
*	Physics Controller Object
*/
	var Physics = function() {

		var apply = function(entity, map, delta) {
			
			// apply natural forces
			naturalForces(entity, delta);

			// if entity to map collision
			if(entity.collide[0]) {
				entityToMap(entity, map);
			}

			// evaluate collisionside
			collisionSide.evaluate(entity);

			// reset movement variables
			entity.m_vx = 0;
			entity.m_vy = 0;

		};

		var naturalForces = function(entity, delta) {
			var gravity = 1;
			var airResistance = 1;

			var vx = entity.vx;
			var vy = entity.vy;
			var weight_x = entity.weight_x;
			var weight_y = entity.weight_y;
			//var m_vx = entity.m_vx;
			//var m_vy = entity.m_vy;

			var dragY = 0;
			var dragX = 0;

			// apply gravity y
			if(weight_y) {
				vy += weight_y*(gravity/delta);
			}

			// apply gravity x
			if(weight_x) {
				vx += weight_x*(gravity/delta);
			}

			// calc drag
			//dragY = Math.pow(vy/delta, 2)*airResistance;
			//dragX = Math.pow(vx/delta, 2)*airResistance;

			// apply natural forces
			entity.vx = vx - dragX;
			entity.vy = vy - dragY;

		};

		var collisionSide = function() {
			var l, r, b, t;

			var reset = function() {
				l = false;
				r = false;
				t = false;
				b = false;
			};

			var left = function() {
				l = true;
			};

			var right = function() {
				r = true;
			};

			var top = function() {
				t = true;
			};

			var bottom = function() {
				b = true;
			};

			var evaluate = function(entity) {
				if(l || r) {
					entity.vx = 0;
				}

				if(b) {
					entity.vy = 0;
					Events.fire('physics:collSideBottom');
				}

				if(t) {
					entity.vy = 0;
					Events.fire('physics:collSideTop');
				}

				reset();
			};

			reset();

			return {
				left: left,
				right: right,
				top: top,
				bottom: bottom,
				evaluate: evaluate
			};
		}();

		var entityToMap = function(entity, map) {
			
			var x = entity.x;
			var y = entity.y;
			var vx = entity.vx;
			var vy = entity.vy;
			var ew = entity.width;
			var eh = entity.height;
			var tilesize = map.tilesize;
			var ewt = Math.ceil(ew/tilesize);
			var eht = Math.ceil(eh/tilesize);
			var width = map.width;
			var data = map.data;
			var xRadius = 0;
			var yRadius = 0;
			var collArray = [];
			var collX = 0;
			var collY = 0;
			var result, i,j, l, coll, down, right;
			var tempX, tempY, dataArrPos;

			// apply movement
			vx += entity.m_vx;
			vy += entity.m_vy;

			if(!vx && !vy) {
				return;
			}

			// get radius for collision checking and direction
			if(vx >= 0) { // right
				xRadius = Math.ceil((vx+ew)/(tilesize))+ewt;
				right = true;
			}else if(vx < 0) { // left
				xRadius = Math.ceil((-vx)/(tilesize))+ewt;
				right = false;
			}

			if(vy >= 0) { // down
				yRadius = Math.ceil((vy+eh)/(tilesize))+eht;
				down = true;
			}else if(vy < 0) { // up
				yRadius = Math.ceil((-vy)/(tilesize))+eht;
				down = false;
			}

			// set x and y coordinates in tiles of entity Position
			collY = Math.floor((y+vy)/tilesize);
			collX = Math.floor((x+vx)/tilesize);

			if(right && down) { // right and down

				// create array of collision checks
				for(j=xRadius-1;j>=0;j-=1) {
					for(i=yRadius-1;i>=0;i-=1) {
						if(j>=ewt || i>=eht) {
							tempX = collX + j;
							tempY = collY + i;
							dataArrPos = tempY*width+tempX;
							if(data[dataArrPos]) {
								collArray.push([tempX,tempY]);
							}
						}
					}
				}

				// check for collision and make correction
				l = collArray.length;
				for(i=0;i<l;i+=1) {
					coll = collArray[i];
					result = downRight(coll[0], coll[1], tilesize, x, y, vx, vy, eh, ew, data, width);
					if(result) {
						if(result[0]) {
							vx += result[0];
							collisionSide.right();
						}else {
							vy += result[1];
							collisionSide.bottom();
						}
					}
				}

				entity.x = x+vx;
				entity.y = y+vy;
				return;

			}else if(!right && down) { // left and down

				// create array of collision checks
				collX -= xRadius-ewt;
				for(j=0;j<=xRadius;j+=1) {
					for(i=yRadius-1;i>=0;i-=1) {
						if(j>=ewt || i>=eht) {
							tempX = collX + j;
							tempY = collY + i;
							dataArrPos = tempY*width+tempX;
							if(data[dataArrPos]) {
								collArray.push([tempX,tempY]);
							}
						}
					}
				}

				// check for collision and make correction
				l = collArray.length;
				for(i=0;i<l;i+=1) {
					coll = collArray[i];
					result = downLeft(coll[0], coll[1], tilesize, x, y, vx, vy, eh, ew, data, width);
					if(result) {
						if(result[0]) {
							vx += result[0];
							collisionSide.left();
						}else {
							vy += result[1];
							collisionSide.bottom();
						}
					}
				}
				entity.x = x+vx;
				entity.y = y+vy;
				return;
				
			}else if(right && !down) { // right and up

				// create array of collision checks
				collY -= yRadius-eht;
				for(j=xRadius-1;j>=0;j-=1) {
					for(i=0;i<=yRadius;i+=1) {
						if(j>=ewt || i>=eht) {
							tempX = collX + j;
							tempY = collY + i;
							dataArrPos = tempY*width+tempX;
							if(data[dataArrPos]) {
								collArray.push([tempX,tempY]);
							}
						}
					}
				}

				// check for collision and make correction
				l = collArray.length;
				for(i=0;i<l;i+=1) {
					coll = collArray[i];
					result = upRight(coll[0], coll[1], tilesize, x, y, vx, vy, eh, ew, data, width);
					if(result) {
						if(result[0]) {
							vx += result[0];
							collisionSide.right();
						}else {
							vy += result[1];
							collisionSide.top();
						}
					}
				}

				entity.x = x+vx;
				entity.y = y+vy;
				return;

			}else { // left and up

				// create array of collision checks
				collX -= xRadius-ewt;
				collY -= yRadius-eht;
				for(j=0;j<=xRadius;j+=1) {
					for(i=0;i<=yRadius;i+=1) {
						if(j>=ewt || i>=eht) {
							tempX = collX + j;
							tempY = collY + i;
							dataArrPos = tempY*width+tempX;
							if(data[dataArrPos]) {
								collArray.push([tempX,tempY]);
							}
						}
					}
				}

				// check for collision and make correction
				l = collArray.length;
				for(i=0;i<l;i+=1) {
					coll = collArray[i];
					result = upLeft(coll[0], coll[1], tilesize, x, y, vx, vy, eh, ew, data, width);
					if(result) {
						if(result[0]) {
							vx += result[0];
							collisionSide.left();
						}else {
							vy += result[1];
							collisionSide.top();
						}
					}
				}
				entity.x = x+vx;
				entity.y = y+vy;
				return;
			}


			entity.x = x+vx;
			entity.y = y+vy;
			
		};

		var createCollisionArray = function(i,j,a,b, width, collX, collY) {
			var collArray = [], temp;

			for(i;i<=a;i+=1) {
				temp = arrPos+(width*i);
				for(j;j<=b;j+=1) {
					temp += 1;
					if(collArray[temp]) {
						collArray.push([collX+j, collY+i]);
					}
				}
			}

			return collArray;
		};

		var downRight = function(bxt, byt, size, x, y, vx, vy, eh, ew, data, mapwidth) {
			var bx = bxt*size;
			var by = byt*size;
			var ax = x+vx;
			var ay = y+vy;
			var awx = ax+ew;
			var ahy = ay+eh;
			var bwx = bx+size;
			var bhy = by+size;
			var slope, xCorrect, yCorrect, left = 0, above = 0;

			var xwx = x+ew;
			var yhy = y+eh;

			if(collision(ax, ay, awx, ahy, bx, by, bwx, bhy)) {

				// calc correction
				xCorrect = awx-bx;
				yCorrect = ahy-by;

				// check if collisiontile is above or left from current collisiontile
				// Order of checks: x 3
				// 					2 1
				// Because of this order, the following rule is correct. Just check it.
				if(bxt > 0) { 
					// If a collision tile is left from the current collision, only a
					//  y-correction makes sense. Trust me.
					left = data[byt*mapwidth+bxt-1];
				}
				if(byt > 0) {
					// Same for top.
					above = data[(byt-1)*mapwidth+bxt];
				}

				// go through different cases and decide how to correct position
				if(left) {
					return [0,-yCorrect];
				}else if(above) {
					return [-xCorrect,0];
				}else if((yhy>by && yhy<bhy) || (y>by && y<bhy)) {
					return [-xCorrect,0];
				}else if((xwx>bx && xwx<=bwx) || (x>=bx && x<=bwx)) {
					return [0,-yCorrect];
				}else {
					slope = vy/vx;
					slope2 = yCorrect/xCorrect;
					if(slope<slope2) {
						return [-xCorrect,0];
					}else {
						return [0,-yCorrect];
					}
				}
			}
			return false;

		};

		var downLeft = function(bxt, byt, size, x, y, vx, vy, eh, ew, data, mapwidth) {
			var bx = bxt*size;
			var by = byt*size;
			var ax = x+vx;
			var ay = y+vy;
			var awx = ax+ew;
			var ahy = ay+eh;
			var bwx = bx+size;
			var bhy = by+size;
			var slope, xCorrect, yCorrect, right = 0, above = 0;

			var xwx = x+ew;
			var yhy = y+eh;

			if(collision(ax, ay, awx, ahy, bx, by, bwx, bhy)) {

				// calc correction
				xCorrect = bwx-ax;
				yCorrect = ahy-by;

				// check if collisiontile is above or left from current collisiontile
				if(bxt > 0) {
					right = data[byt*mapwidth+bxt+1];
				}
				if(byt > 0) {
					above = data[(byt-1)*mapwidth+bxt];
				}
				
				// go through different cases which decide how to correct position
				if(right) {
					return [0,-yCorrect];
				}else if(above) {
					return [xCorrect,0];
				}else if((yhy>by && yhy<bhy) || (y>by && y<bhy)) {
					return [xCorrect,0];
				}else if((xwx>bx && xwx<bwx) || (x>bx && x<bwx)) {
					return [0,-yCorrect];
				}else {
					slope = -vy/vx;
					slope2 = yCorrect/xCorrect;
					//console.log('slope1: ' + slope + ' slope2: ' + slope2);
					if(slope<slope2) {
						return [xCorrect,0];
					}else {
						return [0,-yCorrect];
					}
				}
			}
			return false;
		};

		var upRight = function(bxt, byt, size, x, y, vx, vy, eh, ew, data, mapwidth) {
			var bx = bxt*size;
			var by = byt*size;
			var ax = x+vx;
			var ay = y+vy;
			var awx = ax+ew;
			var ahy = ay+eh;
			var bwx = bx+size;
			var bhy = by+size;
			var slope, xCorrect, yCorrect, below = 0, left = 0;

			var xwx = x+ew;
			var yhy = y+eh;

			if(collision(ax, ay, awx, ahy, bx, by, bwx, bhy)) {
				// calc correction
				xCorrect = awx-bx;
				yCorrect = bhy-ay;

				// check if collisiontile is above or left from current collisiontile
				if(bxt > 0) {
					left = data[byt*mapwidth+bxt-1];
				}

				below = data[(byt+1)*mapwidth+bxt];
				
				// go through different cases which decide how to correct position
				if(left) {
					return [0,yCorrect];
				}else if(below) {
					return [-xCorrect,0];
				}else if((yhy>=by && yhy<=bhy) || (y>=by && y<bhy)) {
					return [-xCorrect,0];
				}else if((xwx>=bx && xwx<=bwx) || (x>=bx && x<bwx)) {
					return [0,yCorrect];
				}else {
					slope = vy/vx;
					slope2 = yCorrect/xCorrect;
					if(slope<slope2) {
						return [-xCorrect,0];
					}else {
						return [0,yCorrect];
					}
				}
			}
			return false;
		};

		var upLeft = function(bxt, byt, size, x, y, vx, vy, eh, ew, data, mapwidth) {
			var bx = bxt*size;
			var by = byt*size;
			var ax = x+vx;
			var ay = y+vy;
			var awx = ax+ew;
			var ahy = ay+eh;
			var bwx = bx+size;
			var bhy = by+size;
			var slope, xCorrect, yCorrect, right = 0, below = 0;

			var xwx = x+ew;
			var yhy = y+eh;

			window.d = data;

			if(collision(ax, ay, awx, ahy, bx, by, bwx, bhy)) {
				xCorrect = bwx-ax;
				yCorrect = bhy-ay;
				
				// check if collisiontile is above or left from current collisiontile
				if(bxt < mapwidth) {
					var pos = byt*mapwidth+bxt+1;
					right = data[byt*mapwidth+bxt+1];
				}
				
				below = data[(byt+1)*mapwidth+bxt];
				
				// go through different cases which decide how to correct position
				if(right) {
					return [0,yCorrect];
				}else if(below) {
					return [xCorrect,0];
				}else if((yhy>=by && yhy<=bhy) || (y>=by && y<bhy)) {
					return [xCorrect,0];
				}else if((xwx>=bx && xwx<=bwx) || (x>=bx && x<bwx)) {
					return [0,yCorrect];
				}else {
					slope = vy/vx;
					slope2 = yCorrect/xCorrect;
					if(slope<slope2) {
						return [xCorrect,0];
					}else {
						return [0,yCorrect];
					}
				}
			}
			return false;
		};

		var collision = function(ax, ay, awx, ahy, bx, by, bwx, bhy) {
			if(ahy>by && ay<bhy) {
				if(awx>bx && ax<bwx) {
					return true;
				}
			}
			return false;
		};
		
		var entityToEntity = function() {

		};

		return {
			apply: apply
		};

	}();


/*--------------------------------------------------------------------------------------------------------->
*	Keys Model Object
*/
	var Keys =  {
		pressed: {},

		KEYS: {
			a: 65,
			w: 87,
			d: 68,
			s: 83
		},

		init: function() {
			var that = this;
			window.addEventListener('keyup', function(event) { that.onKeyup(event); }, false);
			window.addEventListener('keydown', function(event) { that.onKeydown(event); }, false);
		},

		isDown: function(keyName) {
			var keyCode = this.KEYS[keyName];
			return this.pressed[keyCode];
		},
	
		onKeydown: function(event) {
			this.pressed[event.keyCode] = true;
		},
	
		onKeyup: function(event) {
			delete this.pressed[event.keyCode];
		}
	};

	Keys.init();


/*--------------------------------------------------------------------------------------------------------->
*	Renderer View Object
*/
	var Renderer = function() {

		var renderEntity = function(viewport, entity) {
			var x = entity.x;
			var y = entity.y;
			var viewx = viewport[0];
			var viewy = viewport[1];
			var width = entity.width;
			var height = entity.height;
			var app = entity.appearance;

			if(x+width > viewx && y+height > viewy && app.isSet) {
				if(x < viewx + viewport[4] && y < viewy+viewport[5]) {
					var currentTile = app.status;
					var img = app.sprite.image;
					var tilepositions = app.sprite.LUT;
					var temp = tilepositions[currentTile];
					var sx = temp[0];
					var sy = temp[1];
					var vx = x-viewx;
					var vy = y-viewy;

					entity.context.drawImage(img, sx, sy, width, height, vx, vy, width, height);
				}
			}
		};

		var renderEntities = function(viewport, entities) {
			var i, l = entities.length, entity;

			for(i=0;i<l;i+=1) {
				entity = entities[i];
				if(entity.show) {
					renderEntity(viewport, entity);
				}
			}
		};

		var renderTileLayer = function(viewport, layer) {
			var x,y, sx, sy, pos, val, temp;
			var data = layer.data;
			var ctx = layer.ctx;
			var tileset = layer.tileset;
			var LUT = tileset.LUT;
			var tileheight = tileset.tileheight;
			var tilewidth = tileset.tilewidth;
			var layerwidth = layer.width;
			var layerheight = layer.height;
			var xLength = data.length;
			var xStart = Math.floor(viewport[0]/tilewidth);
			var yStart = Math.floor(viewport[1]/tileheight);
			var xEnd = xStart+Math.floor((viewport[4]+tilewidth-1)/tilewidth)+1; // komisch das +1, aber muss wohl sein?
			var yEnd = yStart+Math.floor((viewport[5]+tileheight-1)/tileheight);

			if(xEnd > layerwidth) {
				xEnd = layerwidth;
			}

			if(yEnd > layerheight) {
				yEnd = layerheight;
			}

			ctx.clearRect(0,0,ctx.width, ctx.height);

			for(x=xStart;x<xEnd;x+=1) {
				for(y=yStart;y<yEnd;y+=1) {
					pos = x+(layerwidth*y);
					val = data[pos];
					if(val !== 0 && val !== undefined) {
						temp = LUT[val-1];
						sx = temp[0];
						sy = temp[1];
						ctx.drawImage(tileset.image, sx, sy, tilewidth, tileheight, x*tilewidth-viewport[0], y*tileheight-viewport[1], tilewidth, tileheight);
					}
				}
			}
		};

		var renderImageLayer = function(v, layer) {
			var sx,sy,width,height,x,y,i,l;
			var vwidth = v[4];
			var vheight = v[5];
			var image = layer.asset.image;
			var ctx = layer.ctx;
			var fixed = layer.fixedToViewport;
			var imgs = layer.images;
			var vx = v[0];
			var vy = v[1];
			var diff,xw,yh,vxw,vyh, draw,swidth,sheight,img,data, repeat_x, repeat_y;
			var parallax_x = layer.parallax_x;
			var parallax_y = layer.parallax_y;

			ctx.clearRect(0,0,ctx.width, ctx.height);
			l = imgs.length;
				
			// go through all images of the layer
			for(i=0;i<l;i+=1) {
				img = imgs[i];
				data = img.data;
				sx = data[0];
				sy = data[1];
				swidth = data[2];
				sheight = data[3];
				width = data[4];
				height = data[5];
				x = data[6];
				y = data[7];

				if(fixed) {
					ctx.drawImage(image,sx,sy,swidth,sheight,x,y,swidth,sheight);				
				}else {

					yh = y+height;
					xw = x+width;
					vxw = vx+vwidth;
					vyh = vy+vheight;
					draw = false;
					overflow_x = img.overflow_x;
					overflow_y = img.overflow_y;
					repeat_x = img.repeat_x;
					repeat_y = img.repeat_y;

					var drawImage = function() {
						var tempx = x;
						var tempy = y;
						var temp_swidth = swidth;
						var temp_sheight = sheight;
						var tempsx = sx;
						var tempsy = sy;

						// image overlaps right side of viewport
						if(xw > vxw) {
							diff = (x+temp_swidth)-(vx+vwidth);
							temp_swidth = temp_swidth-diff;
						}

						// image ovrlaps bottom of viewport
						if(yh > vyh) {
							diff = (y+temp_sheight)-(vy+vheight);
							temp_sheight = temp_sheight-diff;
						}

						// image ovrlaps top of viewport
						if(y < vy) {
							diff = vy-y;
							tempsy = sy+diff;
							temp_sheight = temp_sheight-diff;
							tempy = 0;
						}else {
							tempy = tempy-vy;
						}

						// image overlaps left side of viewport 
						if(x < vx) {
							diff = vx-x;
							tempsx = sx+diff;
							temp_swidth = temp_swidth-diff;
							tempx = 0;
						}else {
							tempx = tempx-vx;
						}

						// draw on canvas
						ctx.drawImage(image,tempsx,tempsy,temp_swidth,temp_sheight,tempx,tempy,temp_swidth,temp_sheight);
					};


					if(parallax_x) {
						x = Math.round(x - vx*parallax_x/10);
					}

					if(parallax_y) {
						y = Math.round(y - vy*parallax_y/10);
					}

					// if no repeat and image is smaller than width or height
					//  or if repeat and image is greater than width or height
					if((!repeat_x && width > swidth) || (overflow_x && width < swidth)) {
						width = swidth;
					}

					if(!overflow_x && width < swidth) {
						swidth = width;
					}

					if((!repeat_y && height > sheight) || (overflow_y && height < sheight)) {
						height = sheight;
					}

					if(overflow_y && height < sheight) {
						sheight = height;
					}


					// check if image overlaps with viewport
					// Make corrections in special cases
					var checkForOverlap = function() {
						if(yh>vy && y<vyh) {
							if(xw>vx && x<vxw) {
								draw = true;
							}
						}
					};

					if((overflow_x || overflow_y) && repeat_x && repeat_y) {
							x = x + Math.floor((vx-x)/swidth)*swidth;
							width = vx+vwidth-x;
							xw = x+width;
							y = y + Math.floor((vy-y)/sheight)*sheight;
							height = vy+vheight-y;
							yh = y+height;
							draw = true;
					}else if(overflow_x && repeat_x) {
						if(yh>vy && y<vyh) {
								draw = true;
								x = x + Math.floor((vx-x)/swidth)*swidth;
								width = vx+vwidth-x;
								xw = x+width;
						}
					}else if(overflow_y && repeat_y) {
						if(xw>vx && x<vxw) {
								draw = true;
								y = y + Math.floor((vy-y)/sheight)*sheight;
								height = vy+vheight-y;
								yh = y+height;
						}
					}else {
						checkForOverlap();
					}

					// If image collides with current view
					if(draw) {
						var count_x = 1;
						var count_y = 1;
						var j,z,tempx;

						// if no overflow, then stay in bounds
						if(!overflow_x && width < swidth) {
							swidth = width;
						}
						if(!overflow_y && height < sheight) {
							sheight = height;
						}

						if(repeat_x) {
							count_x = Math.ceil(width/swidth);
						}

						if(repeat_y) {
							count_y = Math.ceil(height/sheight);
						}

						tempx = x;
						for(j=0;j<count_y;j+=1) {
							for(z=0;z<count_x;z+=1) {
								drawImage();
								x += swidth;
							}
							y += sheight;
							x = tempx;
						}
						
					}
				}
			}
		};

		var renderLayers = function(viewport, layers) {
			var i, l = layers.length, layer = {};

			for(i=0;i<l;i+=1) {
				layer = layers[i];
				if(layer.type === 'tileLayer') {
					renderTileLayer(viewport, layer);
				}else if(layer.type === 'imageLayer') {
					renderImageLayer(viewport, layer);
				}
			}
		};

		return {
			renderLayers: renderLayers,
			renderEntities: renderEntities
		};
	}();


/*--------------------------------------------------------------------------------------------------------->
*	Entity Extensions Collection Object
*/


	var EntityExtensions = function() {
		
		var pub = {}, extensions = {};

		extensions.viewable = function() {
				this.appearance = {};
				this.zindex = 0;
				this.show = true;
				var that = this;

				this.appearances = function(){
					var data = {};

					var addAnimation = function(name, spritename, startTileposition, endTileposition, ms) {
						data[name] = {
							isSet: true,
							sprite: that.assets[spritename],
							start: startTileposition,
							end: endTileposition,
							ms: ms,
							lastMod: 0,
							status: startTileposition
						};
					};

					var addStatic = function(name, spritename, tileposition) {
						addAnimation(name, spritename, tileposition, tileposition, 0);
					};

					var set = function(name) {
						that.appearance = data[name];
					};

					return {
						addAnimation: addAnimation,
						addStatic: addStatic,
						set: set
					};
				}();

				this.render = function(viewport, time) {
					Renderer.renderEntity(viewport, this);
				};
		};

		extensions.moveable = function() {
			/* todo */
		};

		extensions.interactable = function() {
			/* todo */
		};

		extensions.controllable = function() {
			this.keyDown = function(keyName) {
				return Keys.isDown(keyName);
			};
		};

		pub.applyDependencies = function(name, that) {
			var extension = extensions[name];
			if(typeof extension === "function") {
				extension.call(that);
			}
		};

		return pub;
	
	}();


/*--------------------------------------------------------------------------------------------------------->
*	Entity Model Constructor
*/
	var Entity = function(entityData) {
		// By doing it this way, the entityData.prototype is not lost but also returned.
		var entity = new entityData();
		var def = {};

		this.initFunctions = [];
		this.x = 0;
		this.y = 0;
		this.vx = 0;
		this.vy = 0;
		this.m_vy = 0;
		this.m_vx = 0;
		this.weight_x = 0;
		this.weight_y = 0;
		this.width = 0;
		this.height = 0;
		this.dependencies = [];
		this.update = function(){};
		this.initialize = function(){};
		this.sprites = [];
		this.bus = Events;

		Utils.extend(this,entity);

	};


/*--------------------------------------------------------------------------------------------------------->
*	Asset Model Constructor
*/
	var Asset = function(assetData) {
		this.image = null;
		this.url = "";
		this.height = 0;
		this.width = 0;
		this.name = "";

		if(assetData.type === "tileset" || assetData.type === "sprite") {
			this.margin = 0;
			this.spacing = 0;
			this.tileheight = 16;
			this.tilewidth = 16;
			this.LUT = [];
		}

		Utils.extend(this, assetData);

	};

	Asset.prototype.init = function() {
			this.calcLUT();
	};

	Asset.prototype.calcLUT = function() {
		if(this.type === 'tileset' || this.type === "sprite") {
			var LUT = [], maxW = 0, maxH = 0, i,j, count = 0;

			maxW = Math.floor((this.width-2*this.margin)/(this.tilewidth+this.spacing));
			maxH = Math.floor((this.height-2*this.margin)/(this.tileheight+this.spacing));

			for(i=0;i<maxW;i+=1) {
				for(j=0;j<maxH;j+=1) {
					LUT[count] = [this.margin+(i*(this.tilewidth+this.spacing)), this.margin+(j*(this.tileheight+this.spacing))];
					count += 1;
				}
			}

			this.LUT = LUT;
		}
	};


/*--------------------------------------------------------------------------------------------------------->
*	TileLayer Model Constructor
*/
	var TileLayer = function(tileLayerData) {
		this.data = [];
		this.name = "";
		this.opacity = 1;
		this.visible = true;
		this.width = 0;
		this.x = 0;
		this.y = 0;
		this.tilesetName = "";
		this.tileset = null;
		this.type = "tileLayer";
		this.zindex = 0;

		var property, type;

		if(Object.prototype.toString.call(tileLayerData) === '[object Object]') {
			for(property in tileLayerData) {
				type = Object.prototype.toString.call(tileLayerData[property]);

				if(Object.prototype.toString.call(this[property]) === type && type !== '[object Undefined]') {
					this[property] = tileLayerData[property];
				}
			}
		}
	};


/*--------------------------------------------------------------------------------------------------------->
*	CollisionLayer Model Constructor
*/
	var CollisionLayer = function(collisionLayerData) {
		this.data = [];
		this.name = "";
		this.width = 0;
		this.tilesize = 16;
		this.x = 0;
		this.y = 0;

		var property, type;

		if(Object.prototype.toString.call(collisionLayerData) === '[object Object]') {
			for(property in collisionLayerData) {
				type = Object.prototype.toString.call(collisionLayerData[property]);

				if(Object.prototype.toString.call(this[property]) === type && type !== '[object Undefined]') {
					this[property] = collisionLayerData[property];
				}
			}
		}
	};


/*--------------------------------------------------------------------------------------------------------->
*	ImageLayer Model Constructor
*/
	var ImageLayer = function(imageLayerData) {
		// layerObj holder
		this.images = [];

		// options
		this.asset = '';
		this.fixedToViewport = false;
		this.width = 0;
		this.height = 0;
		this.parallax_x = 0;
		this.parallax_y = 0;
		this.zindex = 0;
		this.parallax_data = [0,0];

		Utils.extend(this, imageLayerData);
	};

	ImageLayer.addImage = function() {
		var img = {
			data: [0,0,0,0,0,0,0,0], // sx, sy, swidth, sheight, x, s, width, height
			repeat_x: 0,
			repeat_y: 0,
			imageOverflow_x: false,
			imageOverflow_y: false
		}

		Utils.extend(img, obj);

		this.images = img;
	};


/*--------------------------------------------------------------------------------------------------------->
*	AssetCollection Model Constructor
*/

	var AssetCollection = function() {
		this.assets = {};
	};

	AssetCollection.prototype.addAsset = function(name, asset) {
		asset.name = name;
		this.assets[name] = asset;
	};

	AssetCollection.prototype.getAssets = function() {
		return this.assets;
	};

	AssetCollection.prototype.initAssets = function() {
		var asset;

		for(asset in this.assets) {
			this.assets[asset].init();
		}
	};


/*--------------------------------------------------------------------------------------------------------->
*	GlobalAssetCollection Object
*/

	var GlobalAssetCollection = new AssetCollection();


/*--------------------------------------------------------------------------------------------------------->
*	EntityBlueprintCollection Model Constructor
*/
	var EntityBlueprintCollection = function(assetCollection) {
		this.assetCollection = assetCollection;
		this.blueprints = {};
	};

	EntityBlueprintCollection.prototype.addEntityBlueprint = function(blueprint) {
		var name = blueprint.name;

		this.blueprints[name] = blueprint;
	};


/*--------------------------------------------------------------------------------------------------------->
*	GlobalEntityBlueprintCollection Model
*/

	var GlobalEntityBlueprintCollection = new EntityBlueprintCollection(GlobalAssetCollection);


/*--------------------------------------------------------------------------------------------------------->
*	EntityCollection Model Constructor
*/

	var EntityCollection = function() {
		this.entities = [];
		this.context = {};
		this.assets = {};
	};

	EntityCollection.prototype.createEntity = function(blueprint, args) {
		var entity, i, l, spriteName, deps;
		entity = new Entity(blueprint.data);
		deps = blueprint.dependencies;

		// assign assets
		entity.assets = {};
		l = blueprint.sprites.length;
		for(i=0;i<l;i+=1) {
				spriteName = blueprint.sprites[i];
				entity.assets[spriteName] = this.assets[spriteName];
		}

		// assign context
		entity.context = this.context;

		// assign name
		entity.name = blueprint.name;

		// assign dependencies
		l = deps.length;
		for(i=0;i<l;i+=1) {
			EntityExtensions.applyDependencies(deps[i], entity);
		}

		// initialize
		entity.initialize.apply(entity, args);

		// save
		this.entities.push(entity);
	};

	EntityCollection.prototype.getByName = function(name) {
		var i, e = this.entities, j;
		var l = e.length;

		for(i=0;i<l;i+=1) {
			j = e[i];
			if(j.name === name){
				return j;
			}
		}
		return null;
	};

	EntityCollection.prototype.update = function(delta) {
		var i, e = this.entities;
		var l = e.length;

		for(i=0;i<l;i+=1) {
			e[i].update(delta);
		}
	};

	EntityCollection.prototype.physics = function(map, delta) {
		var i, e = this.entities;
		var l = e.length;

		for(i=0;i<l;i+=1) {
			Physics.apply(e[i], map, delta);
		}
	};

	EntityCollection.prototype.render = function(viewport) {
		Renderer.renderEntities(viewport, this.entities);
	};


/*--------------------------------------------------------------------------------------------------------->
*	LayerCollection Model Constructor
*/
	var LayerCollection = function() {
		this.layers = [];
	};

	LayerCollection.prototype.addLayer = function(layer) {
		this.layers.push(layer);
	};

	LayerCollection.prototype.numOfLayers = function() {
		return this.layers.length;
	};

	LayerCollection.prototype.sort = function() {
		this.layers.sort(function(a,b) {
			var az = a.zindex || 0;
			var bz = b.zindex || 0;

			if(az > bz) {
				return 1;
			}else if (az < bz) {
				return -1;
			}

			return 0;
		});
	};

	LayerCollection.prototype.setContexes = function(ctxs) {
		var i, l = this.layers.length;

		this.sort();

		for(i=0;i<l;i+=1) {
			this.layers[i].ctx = ctxs[i];
		}
	};

	LayerCollection.prototype.render = function(viewport) {
		Renderer.renderLayers(viewport, this.layers);
	};

	LayerCollection.prototype.assignAssets = function(assets) {
		var i, l = this.layers.length, tileset = "", asset = "";

		for(i=0;i<l;i+=1) {
			tileset = this.layers[i].tilesetName;
			asset = this.layers[i].asset;
			if(tileset) {
				this.layers[i].tileset = assets[tileset];
			}else if(asset) {
				this.layers[i].asset = assets[asset];
			}
		}
	};

/*--------------------------------------------------------------------------------------------------------->
*	Level Model Constructor
*/

	var Level = function(name, obj) {
		this.layerCollection = new LayerCollection();
		this.collisionLayer = {};
		this.entityCollection = new EntityCollection();
		this.assetCollection = new AssetCollection();
		this.entityBlueprintCollection = new EntityBlueprintCollection(this.assetCollection);
		this.name = name;
		this.pendingEntities = [];

		this.extend(obj);
		window.l = this;
	}

	Level.prototype.extend = function(obj) {
		var i, l, collisionLayer, tileLayer;

		if(Object.prototype.toString.call(obj) === '[object Object]') {

			/* read layers in */
			if(Object.prototype.toString.call(obj.layers) === '[object Array]') {
				var layers = obj.layers, type = "";
				l = layers.length;

				for(i=0;i<l;i+=1) {
					type = layers[i].type;

					if(type === "tileLayer") {
						tileLayer = new TileLayer(layers[i]);
						this.addLayer(tileLayer);
					}else if(type === "collisionLayer") {
						collisionLayer = new CollisionLayer(layers[i]);
						this.addCollisionLayer(collisionLayer);
					}else if(type === "imageLayer") {
						imageLayer = new ImageLayer(layers[i]);
						this.addLayer(imageLayer);
					}
				}
			}

			/* read assets in */
			if(Object.prototype.toString.call(obj.assets) === '[object Array]') {
				var assets = obj.assets, asset, assetName;
				l = assets.length;

				for(i=0;i<l;i+=1) {
					asset = new Asset(assets[i]);
					assetName = assets[i].name;
					this.addAsset(assetName, asset);
				}
			}

			/* read entities in */
			if(Object.prototype.toString.call(obj.entities) === '[object Array]') {
				var entities = obj.entities;
				l = entities.length;

				for(i=0;i<l;i+=1) {
					this.addEntity(entities[i].entity, entities[i].args);
				}
			}
		}
	};

	Level.prototype.addLayer = function(layer) {
		this.layerCollection.addLayer(layer);
	};

	Level.prototype.addCollisionLayer = function(collisionLayer) {
		this.collisionLayer = collisionLayer;
	};

	Level.prototype.addAsset = function(name, asset) {
		this.assetCollection.addAsset(name, asset);
	};

	Level.prototype.addEntityBlueprint = function(entityBlueprint) {
		this.entityBlueprintCollection.addEntityBlueprint(entityBlueprint);
	};

	Level.prototype.createEntity = function(name, args) {
		var blueprint = this.entityBlueprintCollection.blueprints[name];
		this.entityCollection.createEntity(blueprint, args);
	};

	Level.prototype.getAssets = function() {
		var assets = {};

		assets = this.assetCollection.getAssets();

		return assets;
	};

	Level.prototype.numOfLayers = function() {
		return this.layerCollection.numOfLayers() + 1;
	};

	Level.prototype.setContexes = function(ctxs) {
		var num = this.numOfLayers();

		this.layerCollection.setContexes(ctxs);
		this.entityCollection.context = ctxs[0];
	};

	Level.prototype.initAndReset = function() {
		this.assetCollection.initAssets();
		this.importGlobalBlueprints();
		this.createPendingEntities();
	};

	Level.prototype.update = function(delta) {
		this.entityCollection.update(delta);
		this.entityCollection.physics(this.collisionLayer, delta);
		this.createPendingEntities();
	};

	Level.prototype.render = function(viewport) {
		this.layerCollection.render(viewport);
		this.entityCollection.render(viewport);
	};

	Level.prototype.assignAssets = function() {
		var assetName;
		var assets = GlobalAssetCollection.getAssets();
		var localAssets = this.assetCollection.getAssets();

		for(assetName in localAssets) {
			assets[assetName] = localAssets[assetName];
		}

		this.layerCollection.assignAssets(assets);
		this.entityCollection.assets = assets;
	};

	Level.prototype.importGlobalBlueprints = function() {
		var name = "";
		var globalBlueprints = GlobalEntityBlueprintCollection.blueprints;
		var localBlueprints = this.entityBlueprintCollection.blueprints;

		for(name in globalBlueprints) {
			localBlueprints[name] = globalBlueprints[name];
		}

		this.entityBlueprintCollection.blueprints = localBlueprints;
	};

	Level.prototype.createPendingEntities = function() {
		var i, entities = this.pendingEntities, args = [];
		var l = entities.length;

		for(i=0;i<l;i+=1) {
			if(Object.prototype.toString.call(entities[i]) === '[object Array]') {
				if(Object.prototype.toString.call(entities[i][0]) === '[object String]') {
					if(Object.prototype.toString.call(entities[i][1]) === '[object Array]') {
						args = entities[i][1];
					}
					this.createEntity(entities[i][0], args);
				}
			}
		}
		this.pendingEntities = [];
	};

	Level.prototype.addEntity = function(name, args) {
		if(Object.prototype.toString.call(name) === '[object String]') {
			var _args;

			if(Object.prototype.toString.call(args) === '[object Array]') {
				_args = args;
			}

			this.pendingEntities.push([name, _args]);
		}
	};


/*--------------------------------------------------------------------------------------------------------->
*	LevelCollection Model Object
*/
	var LevelCollection = function() {
		var levels = {};

		var getLevel = function(lvlName) {
			if(typeof levels[lvlName] !== "undefined") {
				return levels[lvlName];
			}
			return false;
		};

		var createLevel = function(name, levelData) {
			var lvl;

			if(Object.prototype.toString.call(name) !== '[object String]') {
				return null;
			}

			lvl = getLevel(name);

			// if level already exists
			if(lvl) {
				lvl.extend(levelData);
				return true;
			}

			lvl = new Level(name, levelData);
			levels[name] = lvl;
			InfoMessage.show('Success','Level '+name+'has been added!');
			return true;
		};

		var createLayer = function() {
			/* todo */
		};

		var createEntityBlueprint = function(blueprint) {

			if(Object.prototype.toString.call(blueprint.name) !== '[object String]') {
				return false;
			}

			if(Object.prototype.toString.call(blueprint.scope) === '[object String]' && blueprint.scope !== 'global') {
				var level = this.getLevel(blueprint.scope);
				if(level) {
					level.addEntityBlueprint(blueprint);
					return true;
				}else {
					this.createLevel(blueprint.name);
					level = this.getLevel(blueprint.name);
					level.addEntityBlueprint(blueprint);
					return true;
				}
			}

			GlobalEntityBlueprintCollection.addEntityBlueprint(blueprint);

			return true;
		};

		var createAsset = function() {
			var asset, name, level;

			if(arguments.length === 2) {
				if(Object.prototype.toString.call(arguments[0]) === '[object String]' && Object.prototype.toString.call(arguments[1]) === '[object Object]') {
					asset = new Asset(arguments[1]);
					name = arguments[0];
					GlobalAssetCollection.addAsset(name, asset);
				}
			}else if(arguments.length === 3) {
				if(Object.prototype.toString.call(arguments[0]) === '[object String]' && Object.prototype.toString.call(arguments[1]) === '[object String]' && Object.prototype.toString.call(arguments[2]) === '[object Object]') {
					level = this.getLevel(arguments[0]);
					asset = new Asset(arguments[2]);
					name = arguments[1];
					if(level !== null) {
						level.addAsset(name, asset);
					}
				}
			}
		};

		var createEntity = function(entity) {
			/* if preruntime aaddEntity
			if at runtime createEntity */
			/* todo */

		};

		return {
			createLevel: createLevel,
			createEntityBlueprint: createEntityBlueprint,
			createAsset: createAsset,
			createEntity: createEntity,
			getLevel: getLevel
		};
	}();


/*--------------------------------------------------------------------------------------------------------->
*	Convert Level Data to rigth Format
*/

	var convert = function(type, data) {
		var obj = {};

		if(type === 'tiled') {
			var tilesetArray = [];

			var convertAssets = function(tilesets) {
				var l = tilesets.length, assets = [];

				for(i=0; i < l; i+=1) {
					var asset = {}, set = tilesets[i];

					asset.url = set.image;
					asset.height = set.imageheight;
					asset.width = set.imagewidth;
					asset.margin = set.margin;
					asset.spacing = set.spacing;
					asset.tileheight = set.tileheight;
					asset.tilewidth = set.tilewidth;
					asset.name = set.name;
					asset.type = 'tileset'

					tilesetArray.push([
						set.name,
						set.firstgid,
						set.tilewidth
					]);

					assets.push(asset);
				}

				tilesetArray.sort(function(a,b) {
						return a[1]-b[1];
				});

				return assets;
			};

			var convertLayerData = function(layer) {
				var l = layer.data.length, i, j, arrLength = tilesetArray.length, tilesets = [];

				var existsInTilesets = function(x) {
					var xl = tilesets.length,xi;

					for(xi=0;xi<xl;xi+=1) {
						if(tilesets[xi] == x)
							return true;
					}

					return false;
				};

				// if collision layer take shortcut
				if(layer.type === 'collisionlayer') {
					for(i=0;i<l;i+=1) {
						if(layer.data[i] > 0) {
							layer.data[i] = 1;
						}
					}
					layer.tilesize = tilesetArray[0][2];
					return layer;
				}

				// find used tilesets
				for(i=0;i<l;i+=1) {
					if(layer.data[i]>0) {
						for(j=0;j<arrLength;j+=1) {
							if(layer.data[i] < tilesetArray[j][1]) {
								if(!existsInTilesets(j-1))
									tilesets.push(j-1);
							}else if(j == arrLength-1) {
								if(!existsInTilesets(j))
									tilesets.push(j);
							}
						}
					}
				}

				// check for errors, if none convert layer
				if(tilesets.length > 1) {
					InfoMessage.show('critical','Error in "'+layer.name+'". Not more than one tileset per layer allowed!');
					return null;
				}else if(tilesets.length === 0) {
					InfoMessage.show('critical','Error in "'+layer.name+'". Could not find matching tileset or layer is empty!');
					return null;
				}else if(tilesets.length === 1) {
					// convert layer data
					if(tilesetArray[tilesets[0]][1] !== 1) {
						for(i=0;i<l;i+=1) {
							if(layer.data[i] > 0) {
								layer.data[i] = layer.data[i]-tilesetArray[tilesets[0]][1]+1;
							}
						}
					}

					// set tilest info
					layer.tilesetName = tilesetArray[tilesets[0]][0];

					return layer;
				}else {
					InfoMessage.show('critical','Error in "'+layer.name+'". Unidentified Error!');
				}

				return null;
			};

			var convertTileLayerData = function(layer, zindex) {
				tilelayer = {};

				tilelayer.data = layer.data;
				tilelayer.name = layer.name;
				tilelayer.opacity = layer.opacity || 1;
				tilelayer.visible = layer.visible || true;
				tilelayer.width = layer.width;
				tilelayer.x = layer.x || 0;
				tilelayer.y = layer.y || 0;
				tilelayer.zindex = layer.zindex;
				tilelayer.tilesetName = layer.tilesetName;

				if(layer.type === 'tilelayer') layer.type = 'tileLayer';
				else if (layer.type === 'collisionlayer') layer.type = 'collisonLayer';
				tilelayer.type = layer.type;

				return tilelayer;
			};

			var convertCollisionlayerData = function(layer) {
				var collisionlayer = {};

				collisionlayer.data = layer.data;
				collisionlayer.name = layer.name;
				collisionlayer.width = layer.width;
				collisionlayer.tilesize = layer.tilesize;
				collisionlayer.type = "collisionLayer";

				return collisionlayer;
			};

			var convertLayers = function(layers) {
				var layer, zindex = 1;
				var l = layers.length;

				for(var i=0; i < l; i+=1) {
					layer = convertLayerData(layers[i]);

					if(layer !== null) {
						if(layer.type === "tilelayer") {
							layer = convertTileLayerData(layer, zindex);
							zindex-=1;
						} else if(layer.type === "collisionlayer") {
							layer = convertCollisionlayerData(layer);
						}
					}
					layers[i] = layer;
				}
				return layers;
			};

			if(Object.prototype.toString.call(data) === '[object Object]') {
				/* check for assets */
				if(Object.prototype.toString.call(data.tilesets) === '[object Array]') {
					data.tilesets = convertAssets(data.tilesets);
				}
				/* check for layers */
				if(Object.prototype.toString.call(data.layers) === '[object Array]') {
					data.layers = convertLayers(data.layers);
				}

				obj.layers = data.layers;
				obj.assets = data.tilesets;
			}
		}

		return obj;
	};


/*--------------------------------------------------------------------------------------------------------->
*	Controller Event Bus Object	Settings
*/
	var ControllerBus = Utils.mediator();


/*--------------------------------------------------------------------------------------------------------->
*	Settings Object
*/
	var Settings = function() {
		var data = {};

		var init = function() {
			data.viewportWidth = 800;
			data.viewportHeight = 600;
		}();

		var set = function(obj) {
			var setting;

			if(typeof obj == "object"){
				for(setting in obj){
					data[setting] = obj[setting];
					ControllerBus.fire('SettingsUpdated:'+setting);
				}
			}
		};

		var get = function(name) {
			return data[name];
		};

		return {
			set: set,
			get: get
		};
	}();


/*--------------------------------------------------------------------------------------------------------->
*	CanvasManager Controller Object
*/
	var CanvasManager = function() {
		var canvasSettings = {},
			ctxs = [];

		var initLayer = function() {
			var ctx, pos, canvas;
			
			pos = $('.GAME_layers > canvas').size()-1;
			canvas = document.getElementsByClassName("GAME_layers")[0].children[pos];
			ctx = canvas.getContext("2d");
			ctx.width = canvasSettings.width;
			ctx.height = canvasSettings.height;
			ctx.clearRect(0,0,ctx.canvas.width, ctx.canvas.height);
			return ctx;
		};

		var updateSettings = function() {
			canvasSettings.width = Settings.get("viewportWidth");
			canvasSettings.height = Settings.get("viewportHeight");
			$('.GAME_layers').children().css('width', canvasSettings.width).css('height',canvasSettings.height).css('border','1px solid black');
		};

		var addOne = function() {
			var ctx;

			updateSettings();
			$('.GAME_layers').append('<canvas width="'+canvasSettings.width+'"" height="'+canvasSettings.height+'"></canvas>');
			ctx = initLayer();
			ctxs.push(ctx);
		};

		var removeOne = function() {
			$('.GAME_layers').children().last().remove();
			ctxs.pop();
		};

		var setLayersTo = function(num) {
			var length = $('.GAME_layers').children().length;

			while(length>num) {
				removeOne();
				length -= 1;
			}

			while(length < num) {
				addOne();
				length += 1;
			}

			ControllerBus.fire('CanvasManager_canvasesUpdated', ctxs);
		};

		/* listen on ControllerBus */
		ControllerBus.on('CanvasManager.setNumOfCanvases', setLayersTo);
		ControllerBus.on('SettingsUpdated:viewportWidth', updateSettings);
		ControllerBus.on('SettingsUpdated:viewportHeight', updateSettings);
	}();


/*--------------------------------------------------------------------------------------------------------->
*	AssetManager Controller Object
*/
	var AssetManager = function() {
		var downloadQueue = [];
		var loadCount = 0;

		var check = function(asset) {
			if(typeof asset.url === "string") {
				var l = downloadQueue.length, i, exists = false;

				for(i=0;i<l;i+=1) {
					if(downloadQueue[i].name === asset.name)
						exists = true;
				}

				if(!exists) {
					return true;
				}
			}
			return false;
		};

		var queue = function(assets) {
			var assetName, asset;

			for(assetName in assets) {
				asset = assets[assetName];

				if(check(asset)) {
					asset.errorCount = 0;
					downloadQueue.push(asset);
				}
			}
		};

		var done = function() {
			if(loadCount === downloadQueue.length) {
				loadCount = 0;
				ControllerBus.fire('AssetManager_done');
			}
		};

		var loadFunction = function(e) {
			loadCount += 1;
			var progress = loadCount/downloadQueue.length;
			ControllerBus.fire('AssetManager_progress', progress);
			done();
		};

		var errorFunction = function(e) {
			console.log(e);
			InfoMessage.show('critical','Could not load Asset');
			/* fire error event */
		};

		var abortFunction = function(e) {
			InfoMessage.show('critical','Could not load Asset');
			/* fire error event*/
		};

		var downloadAll = function() {
			var l = downloadQueue.length, i;

			if(l === 0) {
				done();
			}

			for(i=0;i<l;i+=1) {
				var asset = downloadQueue[i];
				var path = asset.url;
				var img = new Image();

				img.addEventListener("load", loadFunction, false);
				img.addEventListener("error", errorFunction, false);
				img.addEventListener("abort", abortFunction, false);

				img.src = path;
				asset.image = img;
			}
		};

		/* listen on ControllerBus */
		ControllerBus.on('AssetManager.queue', queue);
		ControllerBus.on('AssetManager.downloadAll', downloadAll);
	}();


/*--------------------------------------------------------------------------------------------------------->
*	Viewport Controller Object
*/

	var Viewport = {
		data: [0,0,0,0,500,200], //x,y,?,?,width,height

		initialize: function(currentLevel) {
			this.data[4] = Settings.get('viewportWidth');
			this.data[5] = Settings.get('viewportHeight');
			this.tools.data = this.data;
			this.tools.entities = currentLevel.entityCollection;
			if(typeof this.tools.initialize === 'function') {
				this.tools.initialize();
			}
		},

		update: function(delta) {
			var d = this.data, i, l;
			var u = this.tools.update;

			l = u.length;

			for(i=0;i<l;i+=1) {
				u[i](delta);
			}

			d[0] += d[2];
			d[1] += d[3];

			d[0] = Math.round(d[0]);
			d[1] = Math.round(d[1]); 
		},

		tools: {
			data: [0,0,0,0,0,0],

			update: [],

			startFollow: function(entityName) {
				var entity = this.entities.getByName(entityName);
				var data = this.data;
				var offsetX = data[4]/2;
				var offsetY = data[5]/2;

				var followEntity = function() {
					data[0] = entity.x -offsetX;
					data[1] = entity.y -offsetY;
				};

				this.update.push(followEntity);

			},

			extend: function(obj) {
				Utils.extend(this, obj);
			}

		}
	};


/*--------------------------------------------------------------------------------------------------------->
*	Kernel Controller Object
*/
	var Kernel = function() {
		var pub = {}, currentLevel, assetUrls;

		window.requestAnimFrame = (function(){
			return  window.requestAnimationFrame       ||
					window.webkitRequestAnimationFrame ||
					window.mozRequestAnimationFrame    ||
					window.oRequestAnimationFrame      ||
					window.msRequestAnimationFrame     ||
					function( callback ){
						window.setTimeout(callback, 1000 / 60);
					};
		})();

		pub.loadLevel = function(name) {
			var lvl, assets = {}, assetName, levelAssets, merge = {};

			lvl = LevelCollection.getLevel(name);

			if(lvl !== null){
				currentLevel = lvl;
				levelAssets = currentLevel.getAssets();
				assets = GlobalAssetCollection.getAssets();

				for(assetName in assets) {
					merge[assetName] = assets[assetName];
				}

				for(assetName in levelAssets) {
					merge[assetName] = levelAssets[assetName];
				}

				ControllerBus.fire('AssetManager.queue', merge);
				ControllerBus.fire('AssetManager.downloadAll');
			}

		};

		var assetsLoaded = function() {
			var numOfLayers = currentLevel.numOfLayers();
			ControllerBus.fire('CanvasManager.setNumOfCanvases', numOfLayers);
		};

		var initialize = function(ctxs) {
			GlobalAssetCollection.initAssets();
			currentLevel.setContexes(ctxs);
			currentLevel.assignAssets();
			currentLevel.initAndReset();
			Viewport.initialize(currentLevel);
			startGameLoop();
		};

		var startGameLoop = function() {
			var lvl = currentLevel;
			var viewport = {};
			viewport = Viewport.data;
			var lastT = 0;
			var lastFps = 0;
			var fps = 0;
			var delta = 0;

			var calcfps = function(t) {
				fps += 1;

				if(t-lastFps > 999) {
					$('.fps').html("fps: " +fps);
					lastFps = t;
					fps = 0;
				}
			};

			function loop(t) {
				delta = t-lastT;
				lvl.update(delta);
				Viewport.update(delta);

				lvl.render(viewport, t);
				calcfps(t);

				lastT = t;
				requestAnimFrame(loop);
			}
			
			requestAnimFrame(loop);

		};

		/* listen on ControllerBus */
		ControllerBus.on('AssetManager_done', assetsLoaded);
		ControllerBus.on('CanvasManager_canvasesUpdated', initialize);

		return pub;
	}();


/*--------------------------------------------------------------------------------------------------------->
*	Function for initializing and starting the Kernel
*/
	var start = function() {
		if(typeof this.initialize === "function") {
			this.initialize();
		}
	};

	var pause = function(){
		/* todo */
	};

	var reset = function() {
		/* todo */
	};


/*--------------------------------------------------------------------------------------------------------->
*	Function for initializing the Game.
*/
	var extend = function(obj) {
		var prop;

		for(prop in obj) {
			this[prop] = obj[prop];
		}

		return this;
	};


/*--------------------------------------------------------------------------------------------------------->
*	Return public API
*/
	return {
		createLevel: LevelCollection.createLevel,
		createAsset: LevelCollection.createAsset,
		createEntityBlueprint:	LevelCollection.createEntityBlueprint,
		createLayer: LevelCollection.createLayer,
		createEntity: LevelCollection.createEntity,
		Viewport: Viewport.tools,
		convert: convert,
		extend: extend,
		start: start,
		pause: pause,
		reset: reset,
		Kernel: Kernel
	};
}();